require "import"
import "android.app.*"
import "android.os.*"
import "android.widget.*"
import "android.view.*"
import "android.content.*"
import "android.provider.*"
import "android.media.*"
import "android.graphics.Color"
import "java.io.File"
import "java.util.ArrayList"

activity.setTheme(android.R.style.Theme_DeviceDefault)

function loadAudioFiles()
    local audioFiles = {}
    local projection = {
        MediaStore.Audio.Media._ID,
        MediaStore.Audio.Media.DISPLAY_NAME,
        MediaStore.Audio.Media.DATA
    }
    
    if Build.VERSION.SDK_INT >= 23 then
        local permissions = {
            "android.permission.READ_EXTERNAL_STORAGE",
            "android.permission.WRITE_EXTERNAL_STORAGE"
        }
        activity.requestPermissions(permissions, 1)
    end
    
    local cursor = activity.getContentResolver().query(
        MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
        projection,
        nil,
        nil,
        MediaStore.Audio.Media.DISPLAY_NAME
    )
    
    if cursor then
        while cursor.moveToNext() do
            local name = cursor.getString(cursor.getColumnIndex(MediaStore.Audio.Media.DISPLAY_NAME))
            local path = cursor.getString(cursor.getColumnIndex(MediaStore.Audio.Media.DATA))
            if path and name then
                table.insert(audioFiles, {name = name, path = path})
            end
        end
        cursor.close()
    end
    
    return audioFiles
end

function showSpeedPopupMenu(anchor, setPlaybackSpeed)
    local popup = PopupMenu(activity, anchor)
    local menu = popup.getMenu()
    
    local speedOptions = {
        {"0.5x", 0.5},
        {"1.0x", 1.0},
        {"1.5x", 1.5},
        {"2.0x", 2.0},
        {"3.0x", 3.0},
        {"4.0x", 4.0},
        {"5.0x", 5.0}
    }
    
    for i, option in ipairs(speedOptions) do
        local title, _ = table.unpack(option)
        menu.add(1, i, i, title)
    end
    
    popup.setOnMenuItemClickListener{
        onMenuItemClick = function(item)
            local id = item.getItemId()
            local _, speed = table.unpack(speedOptions[id])
            setPlaybackSpeed(speed)
            anchor.setText("Speed: " .. item.getTitle())
            return true
        end
    }
    
    popup.show()
end

function showPlayerDialog(audioFile, currentIndex)
    local dialog = Dialog(activity)
    dialog.setTitle(audioFile.name)
    
    local dialogLayout = LinearLayout(activity)
    dialogLayout.setOrientation(1)
    dialogLayout.setPadding(20, 20, 20, 20)
    dialogLayout.setGravity(Gravity.CENTER)
    
    -- Track Controls at the top
    local trackLayout = LinearLayout(activity)
    trackLayout.setOrientation(0)
    trackLayout.setGravity(Gravity.CENTER)
    trackLayout.setPadding(0, 0, 0, 20)
    
    local leftTrackBtn = Button(activity)
    leftTrackBtn.setText("⏮ Previous")
    leftTrackBtn.setTextSize(16)
    local leftParams = LinearLayout.LayoutParams(-2, -2)
    leftParams.weight = 1
    leftParams.setMargins(5, 0, 5, 0)
    leftTrackBtn.setLayoutParams(leftParams)
    
    local rightTrackBtn = Button(activity)
    rightTrackBtn.setText("Next ⏭")
    rightTrackBtn.setTextSize(16)
    local rightParams = LinearLayout.LayoutParams(-2, -2)
    rightParams.weight = 1
    rightParams.setMargins(5, 0, 5, 0)
    rightTrackBtn.setLayoutParams(rightParams)
    
    trackLayout.addView(leftTrackBtn)
    trackLayout.addView(rightTrackBtn)
    dialogLayout.addView(trackLayout)
    
    local timeText = TextView(activity)
    timeText.setGravity(Gravity.CENTER)
    timeText.setText("00:00 / 00:00")
    timeText.setTextSize(16)
    dialogLayout.addView(timeText)
    
    local seekBar = SeekBar(activity)
    seekBar.setMax(100)
    dialogLayout.addView(seekBar)
    
    local speedButton = Button(activity)
    speedButton.setText("Speed: 1.0x")
    speedButton.setPadding(20, 10, 20, 10)
    dialogLayout.addView(speedButton)
    
    local controlsLayout = LinearLayout(activity)
    controlsLayout.setOrientation(0)
    controlsLayout.setGravity(Gravity.CENTER)
    controlsLayout.setPadding(0, 20, 0, 20)
    
    local rewindBtn = Button(activity)
    rewindBtn.setText("⏪ 10s")
    rewindBtn.setTextSize(20)
    
    local playBtn = Button(activity)
    playBtn.setText("▶")
    playBtn.setTextSize(24)
    
    local forwardBtn = Button(activity)
    forwardBtn.setText("10s ⏩")
    forwardBtn.setTextSize(20)
    
    controlsLayout.addView(rewindBtn)
    controlsLayout.addView(playBtn)
    controlsLayout.addView(forwardBtn)
    
    dialogLayout.addView(controlsLayout)
    dialog.setContentView(dialogLayout)
    
    local mediaPlayer = MediaPlayer()
    pcall(function()
        mediaPlayer.setDataSource(audioFile.path)
        mediaPlayer.prepare()
        mediaPlayer.start()
        playBtn.setText("⏸")
    end)
    
    local currentSpeed = 1.0
    
    -- Track control functionality
    leftTrackBtn.onClick = function()
        local prevIndex = currentIndex - 1
        if prevIndex >= 1 then
            mediaPlayer.release()
            dialog.dismiss()
            showPlayerDialog(filteredFiles[prevIndex], prevIndex)
        else
            Toast.makeText(activity, "No previous track available", Toast.LENGTH_SHORT).show()
        end
    end
    
    rightTrackBtn.onClick = function()
        local nextIndex = currentIndex + 1
        if nextIndex <= #filteredFiles then
            mediaPlayer.release()
            dialog.dismiss()
            showPlayerDialog(filteredFiles[nextIndex], nextIndex)
        else
            Toast.makeText(activity, "No next track available", Toast.LENGTH_SHORT).show()
        end
    end
    
    local function setPlaybackSpeed(speed)
        if mediaPlayer then
            pcall(function()
                if Build.VERSION.SDK_INT >= 23 then
                    local params = mediaPlayer.getPlaybackParams()
                    params.setSpeed(speed)
                    mediaPlayer.setPlaybackParams(params)
                else
                    mediaPlayer.setPlaybackParams(mediaPlayer.getPlaybackParams().setSpeed(speed))
                end
                currentSpeed = speed
            end)
        end
    end
    
    speedButton.onClick = function()
        showSpeedPopupMenu(speedButton, setPlaybackSpeed)
    end
    
    playBtn.onClick = function()
        if mediaPlayer.isPlaying() then
            mediaPlayer.pause()
            playBtn.setText("▶")
        else
            mediaPlayer.start()
            setPlaybackSpeed(currentSpeed)
            playBtn.setText("⏸")
        end
    end
    
    forwardBtn.onClick = function()
        local currentPos = mediaPlayer.getCurrentPosition()
        local newPos = currentPos + 10000
        if newPos > mediaPlayer.getDuration() then
            newPos = mediaPlayer.getDuration()
        end
        mediaPlayer.seekTo(newPos)
    end
    
    rewindBtn.onClick = function()
        local currentPos = mediaPlayer.getCurrentPosition()
        local newPos = currentPos - 10000
        if newPos < 0 then
            newPos = 0
        end
        mediaPlayer.seekTo(newPos)
    end
    
    seekBar.setOnSeekBarChangeListener{
        onProgressChanged = function(seekBar, progress, fromUser)
            if fromUser then
                local duration = mediaPlayer.getDuration()
                local newPosition = (progress * duration) / 100
                mediaPlayer.seekTo(newPosition)
            end
        end
    }
    
    local handler = Handler()
    local function updateProgress()
        if not dialog.isShowing() then
            handler.removeCallbacks(updateProgress)
            return
        end
        
        if mediaPlayer.isPlaying() then
            local currentPosition = mediaPlayer.getCurrentPosition()
            local duration = mediaPlayer.getDuration()
            local progress = (currentPosition * 100) / duration
            
            seekBar.setProgress(progress)
            timeText.setText(string.format("%02d:%02d / %02d:%02d",
                math.floor(currentPosition/1000/60),
                math.floor(currentPosition/1000%60),
                math.floor(duration/1000/60),
                math.floor(duration/1000%60)))
        end
        
        handler.postDelayed(updateProgress, 1000)
    end
    
    handler.post(updateProgress)
    
    dialog.setOnDismissListener{
        onDismiss = function()
            mediaPlayer.release()
            handler.removeCallbacks(updateProgress)
        end
    }
    
    dialog.show()
end

function onCreate(savedInstanceState)
    activity.setTitle("Audio Player")
    
    local layout = LinearLayout(activity)
    layout.setOrientation(1)
    layout.setPadding(16, 16, 16, 16)
    
    local folderButton = Button(activity)
    folderButton.setText("📁 Select Folder")
    layout.addView(folderButton)
    
    local searchLayout = LinearLayout(activity)
    searchLayout.setOrientation(0)
    
    local searchEdit = EditText(activity)
    searchEdit.setHint("Search audio files...")
    local layoutParams = LinearLayout.LayoutParams(-1, -2, 1)
    searchEdit.setLayoutParams(layoutParams)
    searchLayout.addView(searchEdit)
    
    local clearButton = Button(activity)
    clearButton.setText("×")
    clearButton.setVisibility(View.GONE)
    searchLayout.addView(clearButton)
    
    layout.addView(searchLayout)
    
    local audioFiles = loadAudioFiles()
    filteredFiles = audioFiles
    
    local listView = ListView(activity)
    local data = ArrayList()
    
    local adapter = SimpleAdapter(activity,
        data,
        android.R.layout.simple_list_item_1,
        {"name"},
        {android.R.id.text1}
    )
    
    local function updateList()
        data.clear()
        for _, file in ipairs(filteredFiles) do
            local item = HashMap()
            item.put("name", file.name)
            data.add(item)
        end
        adapter.notifyDataSetChanged()
    end
    
    folderButton.onClick = function()
        local folders = {}
        local seen = {}
        
        for _, audioFile in ipairs(audioFiles) do
            local file = File(audioFile.path)
            local parentPath = file.getParent()
            if parentPath and not seen[parentPath] then
                table.insert(folders, {
                    name = File(parentPath).getName(),
                    path = parentPath
                })
                seen[parentPath] = true
            end
        end
        
        local dialog = Dialog(activity)
        dialog.setTitle("Select Folder")
        
        local folderLayout = LinearLayout(activity)
        folderLayout.setOrientation(1)
        
        local folderListView = ListView(activity)
        local folderData = ArrayList()
        
        local folderAdapter = SimpleAdapter(activity,
            folderData,
            android.R.layout.simple_list_item_1,
            {"name"},
            {android.R.id.text1}
        )
        
        local allItem = HashMap()
        allItem.put("name", "📁 All Files")
        folderData.add(allItem)
        
        for _, folder in ipairs(folders) do
            local item = HashMap()
            item.put("name", "📁 " .. folder.name)
            folderData.add(item)
        end
        
        folderListView.setAdapter(folderAdapter)
        folderListView.onItemClick = function(parent, view, position, id)
            if position == 0 then
                filteredFiles = audioFiles
            else
                local selectedFolder = folders[position]
                filteredFiles = {}
                for _, file in ipairs(audioFiles) do
                    if string.find(file.path, selectedFolder.path, 1, true) then
                        table.insert(filteredFiles, file)
                    end
                end
            end
            updateList()
            dialog.dismiss()
        end
        
        folderLayout.addView(folderListView)
        dialog.setContentView(folderLayout)
        dialog.show()
    end
    
    searchEdit.addTextChangedListener{
        onTextChanged = function(s)
            local query = string.lower(tostring(s))
            if #query > 0 then
                clearButton.setVisibility(View.VISIBLE)
                filteredFiles = {}
                for _, file in ipairs(audioFiles) do
                    if string.find(string.lower(file.name), query) then
                        table.insert(filteredFiles, file)
                    end
                end
            else
                clearButton.setVisibility(View.GONE)
                filteredFiles = audioFiles
            end
            updateList()
        end,
        beforeTextChanged = function() end,
        afterTextChanged = function() end
    }
    
    clearButton.onClick = function()
        searchEdit.setText("")
        filteredFiles = audioFiles
        updateList()
    end
    
    listView.setAdapter(adapter)
    listView.onItemClick = function(parent, view, position, id)
        showPlayerDialog(filteredFiles[position + 1], position + 1)
    end
    
    updateList()
    layout.addView(listView)
    
    activity.setContentView(layout)
end

function onDestroy()
    if mediaPlayer then
        mediaPlayer.release()
    end
end